@@ -1,5 +1,8 @@
+from typing import Any, List, Optional
+
 from samtranslator.model import PropertyType, Resource
 from samtranslator.model.types import is_type, is_str
+from samtranslator.utils.types import Intrinsicable
 
 
 class Route53RecordSetGroup(Resource):
@@ -9,3 +12,7 @@ class Route53RecordSetGroup(Resource):
         "HostedZoneName": PropertyType(False, is_str()),
         "RecordSets": PropertyType(False, is_type(list)),
     }
+
+    HostedZoneId: Optional[Intrinsicable[str]]
+    HostedZoneName: Optional[Intrinsicable[str]]
+    RecordSets: Optional[List[Any]]
