@@ -1,3 +1,3 @@
-# flake8: noqa
+__all__ = ["InvalidResourceException", "InvalidDocumentException", "InvalidEventException"]
 
 from samtranslator.model.exceptions import InvalidResourceException, InvalidDocumentException, InvalidEventException
